"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var transformAST_1 = require("./transformAST");
var defaultOptions = {
    target: ts.ScriptTarget.Latest,
    module: ts.ModuleKind.CommonJS
};
function isClassComponent(entry, exportedOnly) {
    if (exportedOnly === void 0) { exportedOnly = true; }
    return (exportedOnly === false || entry.exported)
        && entry.baseType
        && entry.baseType.name.indexOf('Component') > -1
        && entry.methods.some(function (j) { return j.name === 'render'; });
}
function isVarComponent(entry, interfaces, exportedOnly) {
    if (exportedOnly === void 0) { exportedOnly = true; }
    return (exportedOnly === false || entry.exported)
        && entry.kind === 'arrowFunction'
        && entry.arrowFunctionParams.length === 1
        && interfaces.some(function (i) { return i.name === entry.arrowFunctionParams[0]; });
}
function isHocClassComponent(entry, classes) {
    return entry.exported
        && entry.kind === 'callExpression'
        && entry.callExpressionArguments.length === 1
        && classes
            .filter(function (i) { return isClassComponent(i, false); })
            .some(function (i) { return i.name === entry.callExpressionArguments[0]; });
}
function isHocVarComponent(entry, variables, interfaces) {
    return entry.exported
        && entry.kind === 'callExpression'
        && entry.callExpressionArguments.length === 1
        && variables
            .filter(function (i) { return isVarComponent(i, interfaces, false); })
            .some(function (i) { return i.name === entry.callExpressionArguments[0]; });
}
function getInterfaceDoc(entry) {
    return {
        name: entry.name,
        comment: entry.comment,
        members: entry.properties
    };
}
function getClassPropInterface(interfaces, classEntry) {
    if (classEntry.baseType.typeArguments.length === 0) {
        return {
            name: 'none',
            comment: '',
            members: [],
        };
    }
    var propInterfaceName = classEntry.baseType.typeArguments[0];
    return getPropInterface(interfaces, propInterfaceName);
}
function getVarPropInterface(interfaces, varEntry) {
    var propInterfaceName = varEntry.arrowFunctionParams[0];
    return getPropInterface(interfaces, varEntry.arrowFunctionParams[0]);
}
function getPropInterface(interfaces, propInterfaceName) {
    var matchedInterfaces = interfaces.filter(function (j) { return j.name === propInterfaceName; });
    if (matchedInterfaces.length === 0) {
        console.warn("Property interface " + propInterfaceName + " cannot be found");
        return {
            name: propInterfaceName,
            comment: '',
            members: [],
        };
    }
    return getInterfaceDoc(matchedInterfaces[0]);
}
/** Generate documention for all classes in a set of .ts files */
function getFileDocumentation(fileName, options) {
    if (options === void 0) { options = defaultOptions; }
    var components = [];
    var program = ts.createProgram([fileName], options);
    var checker = program.getTypeChecker();
    var sourceFile = program.getSourceFile(fileName);
    var model = transformAST_1.transformAST(program.getSourceFile(fileName), checker);
    var interfaces = model.interfaces, classes = model.classes, variables = model.variables, types = model.types;
    // let's treat types as interfaces here both can include valid props definition
    var allInterfaces = interfaces.concat(types);
    allInterfaces.forEach(function (i) {
        i.properties = i.properties.filter(function (j) { return j.isOwn; });
    });
    var classComponents = classes
        .filter(function (i) { return isClassComponent(i); })
        .map(function (i) { return ({
        name: i.name,
        extends: i.baseType.name,
        comment: i.comment,
        propInterface: getClassPropInterface(allInterfaces, i),
    }); });
    var varComponents = variables
        .filter(function (i) { return isVarComponent(i, allInterfaces); })
        .map(function (i) { return ({
        name: i.name,
        extends: 'StatelessComponent',
        comment: i.comment,
        propInterface: getVarPropInterface(allInterfaces, i),
    }); });
    var hocClassComponents = variables
        .filter(function (i) { return isHocClassComponent(i, classes); })
        .map(function (i) { return ({
        variable: i,
        origin: classes.filter(function (c) { return c.name === i.callExpressionArguments[0]; })[0]
    }); })
        .map(function (i) { return ({
        name: i.variable.name,
        extends: i.variable.callExpressionArguments[0],
        comment: i.variable.comment || i.origin.comment,
        propInterface: getClassPropInterface(allInterfaces, i.origin),
    }); });
    var hocVarComponents = variables
        .filter(function (i) { return isHocVarComponent(i, variables, allInterfaces); })
        .map(function (i) { return ({
        variable: i,
        origin: variables.filter(function (c) { return c.name === i.callExpressionArguments[0]; })[0]
    }); })
        .map(function (i) { return ({
        name: i.variable.name,
        extends: i.variable.callExpressionArguments[0],
        comment: i.variable.comment || i.origin.comment,
        propInterface: getVarPropInterface(allInterfaces, i.origin),
    }); });
    return {
        components: classComponents.concat(varComponents, hocClassComponents, hocVarComponents)
    };
}
exports.getFileDocumentation = getFileDocumentation;
//# sourceMappingURL=getFileDocumentation.js.map