"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var nodeUtils_1 = require("./nodeUtils");
/**
 * Checks if the node is exported.
 */
function isNodeExported(node) {
    // Parse the modifier array for the export keyword. If it is found
    // and the node.parent is a sourcefile, return true
    // This only returns top level exports
    var modifiers = node.modifiers;
    if (modifiers) {
        for (var i = 0; i < modifiers.length; i++) {
            if (modifiers[i].kind === ts.SyntaxKind.ExportKeyword) {
                return node.parent !== undefined && node.parent.kind === ts.SyntaxKind.SourceFile;
            }
        }
    }
    return false;
}
function getType(prop) {
    var unionType = prop.type;
    if (unionType && unionType.types) {
        return {
            type: 'string',
            values: unionType.types.map(function (i) { return i.getText(); }),
        };
    }
    //noinspection TypeScriptUnresolvedFunction
    return {
        type: unionType.getText(),
    };
}
function getMethods(checker, type, classDeclaratinNode) {
    return classDeclaratinNode.members
        .filter(function (x) { return x.name !== undefined; })
        .map(function (i) { return ({ name: i.name ? i.name.getText() : 'unknown' }); });
}
function getProperties(checker, type, parent) {
    var baseTypes = type.getBaseTypes() || [];
    var inheritedProperties = baseTypes
        .reduce(function (acc, bt) { return acc.concat(bt.getProperties().map(function (p) { return p.getName(); })); }, []);
    return type.getProperties()
        .filter(function (i) { return i.valueDeclaration; })
        .map(function (i) {
        if (i.valueDeclaration === undefined || i.valueDeclaration === null) {
            throw Error('The valueDeclaration does not exist');
        }
        var symbol = checker.getSymbolAtLocation(i.valueDeclaration.name);
        var prop = i.valueDeclaration;
        var typeInfo = getType(prop);
        var propertyName = i.getName();
        return {
            name: propertyName,
            isOwn: inheritedProperties.indexOf(propertyName) === -1,
            type: typeInfo.type,
            values: typeInfo.values || [],
            isRequired: !prop.questionToken,
            comment: ts.displayPartsToString(symbol.getDocumentationComment()).trim(),
        };
    });
}
function findAllNodes(rootNode, result) {
    result.push(rootNode);
    ts.forEachChild(rootNode, function (node) {
        findAllNodes(node, result);
    });
}
/**
 * Transform source file AST (abstract syntax tree) to our
 * model (classes, interfaces, variables, methods).
 */
function transformAST(sourceFile, checker) {
    var nodes = [];
    findAllNodes(sourceFile, nodes);
    var variables = nodes
        .filter(function (i) { return i.kind === ts.SyntaxKind.VariableStatement; })
        .map(function (i) { return i; })
        .filter(function (i) { return i.declarationList.declarations
        && i.declarationList.declarations.length === 1
        && i.declarationList.declarations[0].name.kind === ts.SyntaxKind.Identifier
        && i.declarationList.declarations[0].initializer; })
        .map(function (i) {
        var d = i.declarationList.declarations[0];
        var identifier = d.name;
        if (!d.initializer) {
            throw Error('The initializer property does not exist');
        }
        var symbol = checker.getSymbolAtLocation(identifier);
        var arrowFunctionType = 'undefined';
        var kind = 'unknown';
        var varType = checker.getTypeAtLocation(d);
        var initializerType = checker.getTypeAtLocation(d.initializer);
        var initializerFlags = initializerType.flags;
        var arrowFunctionParams = [];
        var callExpressionArguments = [];
        if (!d.initializer) {
            kind = 'unknown';
        }
        else if (d.initializer.kind === ts.SyntaxKind.ArrowFunction) {
            var arrowFunc = d.initializer;
            if (arrowFunc.parameters) {
                arrowFunctionParams = arrowFunc
                    .parameters
                    .map(function (p) { return p.type ? p.type.getText() : 'unknown'; });
            }
            arrowFunctionType = arrowFunc.type ? arrowFunc.type.getText() : 'undefined';
            kind = 'arrowFunction';
        }
        else if (d.initializer.kind === ts.SyntaxKind.FirstLiteralToken) {
            var literal = d.initializer;
            kind = 'literal';
        }
        else if (d.initializer.kind === ts.SyntaxKind.CallExpression) {
            kind = 'callExpression';
            var callExpresson = d.initializer;
            if (callExpresson.arguments) {
                callExpressionArguments = callExpresson.arguments.map(function (i) { return i.getText(); });
            }
        }
        return {
            name: identifier.text,
            exported: isNodeExported(i),
            comment: symbol ? ts.displayPartsToString(symbol.getDocumentationComment()).trim() : '',
            type: varType.symbol ? varType.symbol.getName() : 'unknown',
            kind: kind,
            arrowFunctionType: arrowFunctionType,
            arrowFunctionParams: arrowFunctionParams,
            callExpressionArguments: callExpressionArguments,
            initializerFlags: initializerFlags,
        };
    });
    var interfaces = nodes
        .filter(function (i) { return i.kind === ts.SyntaxKind.InterfaceDeclaration; })
        .map(function (i) { return i; })
        .map(function (i) {
        var symbol = checker.getSymbolAtLocation(i.name);
        var type = checker.getTypeAtLocation(i.name);
        return {
            name: symbol.name,
            comment: ts.displayPartsToString(symbol.getDocumentationComment()).trim(),
            exported: isNodeExported(i),
            properties: getProperties(checker, type, i),
        };
    });
    var types = nodes
        .filter(function (i) { return i.kind === ts.SyntaxKind.TypeAliasDeclaration; })
        .map(function (i) { return i; })
        .map(function (i) {
        var type = checker.getTypeAtLocation(i.name);
        var properties = [];
        if (type.types) {
            type.types.forEach(function (t) {
                var props = t.properties;
                var ownProperties = [];
                if (props) {
                    ownProperties = props
                        .map(function (p) { return p.getName(); });
                }
                properties.push.apply(properties, getProperties(checker, t, i));
                properties
                    .forEach(function (p) { return p.isOwn = ownProperties.indexOf(p.name) > -1; });
            });
        }
        var symbol = checker.getSymbolAtLocation(i.name);
        return {
            name: i.name.getText(),
            properties: properties,
            exported: isNodeExported(i),
            comment: !symbol ? "" : ts.displayPartsToString(symbol.getDocumentationComment())
        };
    });
    var classes = nodes
        .filter(function (i) { return i.kind === ts.SyntaxKind.ClassDeclaration; })
        .map(function (i) { return i; })
        .map(function (i) {
        var symbol = checker.getSymbolAtLocation(i.name);
        var type = checker.getTypeAtLocation(i.name);
        var baseTypes = type.getBaseTypes();
        var baseType = {
            name: 'unknown',
            typeArguments: [],
        };
        if (baseTypes.length) {
            var t = baseTypes[0];
            var typeArguments = nodeUtils_1.navigate(i, ts.SyntaxKind.HeritageClause, ts.SyntaxKind.ExpressionWithTypeArguments);
            if (typeArguments && typeArguments.typeArguments) {
                typeArguments.typeArguments.forEach(function (ta) {
                    var taType = checker.getTypeAtLocation(ta);
                    if (taType && taType.symbol) {
                        var taTypeName_1 = taType.symbol.getName();
                        // check if the type is defined in another file
                        if (interfaces.every(function (int) { return int.name !== taTypeName_1; })
                            && taTypeName_1 !== '__type') {
                            // in that case we need to include the interface explicitly
                            interfaces.push({
                                name: taType.symbol.name,
                                comment: ts.displayPartsToString(taType.symbol.getDocumentationComment()).trim(),
                                exported: true,
                                properties: getProperties(checker, taType, null),
                            });
                        }
                    }
                });
            }
            baseType = {
                name: t.symbol ? t.symbol.getName() : 'unknown',
                typeArguments: typeArguments && typeArguments.typeArguments ?
                    typeArguments.typeArguments.map(function (t) { return t.getText(); }) : []
            };
        }
        return {
            name: symbol.name,
            exported: isNodeExported(i),
            baseType: baseType,
            comment: ts.displayPartsToString(symbol.getDocumentationComment()).trim(),
            methods: getMethods(checker, type, i),
        };
    });
    return {
        classes: classes,
        interfaces: interfaces,
        variables: variables,
        types: types,
    };
}
exports.transformAST = transformAST;
//# sourceMappingURL=transformAST.js.map